package cn.iocoder.yudao.module.order.controller.admin.orderPickup;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.order.vo.orderPickup.*;
import cn.iocoder.yudao.module.order.dal.dataobject.orderPickup.OrderPickupDO;
import cn.iocoder.yudao.module.order.convert.orderPickup.OrderPickupConvert;
import cn.iocoder.yudao.module.order.service.orderPickup.OrderPickupService;

@Validated
@RestController
@Api(tags = "管理后台 - 提货")
@RequestMapping("/ecw/order-pickup")
public class OrderPickupController {

    @Resource
    private OrderPickupService orderPickupService;

    @PostMapping("/create")
    @ApiOperation("创建提货")
    public CommonResult<Long> createOrderPickup(@Valid @RequestBody OrderPickupCreateReqVO createReqVO) {
        return success(orderPickupService.createOrderPickup(createReqVO));
    }

    @PostMapping("/createBatch")
    @ApiOperation("批量提货")
    @Idempotent(timeout = 300)
    public CommonResult<Boolean> createBatchOrderPickup(@Valid @RequestBody List<OrderPickupCreateReqVO> createReqVOList) {
        try {
            orderPickupService.createBatchOrderPickup(createReqVOList);
            return success(true);
        } catch (ServiceException e) {
            return error(e.getCode(),e.getMessage());
        }
    }

    @PostMapping("/createSplitBatch")
    @ApiOperation("分批提货")
    @Idempotent(timeout = 300)
    public CommonResult<Boolean> createSplitBatch(@Valid @RequestBody List<OrderPickupCreateReqVO> createReqVOList) {
        try {
            orderPickupService.createSplitBatch(createReqVOList);
            return success(true);
        } catch (ServiceException e){
            return error(e.getCode(),e.getMessage());
        }
    }

    @PutMapping("/update")
    @ApiOperation("更新提货")
    public CommonResult<Boolean> updateOrderPickup(@Valid @RequestBody OrderPickupUpdateReqVO updateReqVO) {
        orderPickupService.updateOrderPickup(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除提货（撤销提货日志）")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<Boolean> deleteOrderPickup(@RequestParam("id") Long id) {
        orderPickupService.deleteOrderPickup(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得提货")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OrderPickupBackVO> getOrderPickup(@RequestParam("id") Long id) {
        OrderPickupDO orderPickup = orderPickupService.getOrderPickup(id);
        return success(OrderPickupConvert.INSTANCE.convert(orderPickup));
    }

    @GetMapping("/list")
    @ApiOperation("获得提货列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    public CommonResult<List<OrderPickupBackVO>> getOrderPickupList(@RequestParam("ids") Collection<Long> ids) {
        List<OrderPickupDO> list = orderPickupService.getOrderPickupList(ids);
        return success(OrderPickupConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/getPickUpListByOrderNo")
    @ApiOperation("根据订单编号获得分批提货数据")
    @ApiImplicitParam(name = "orderIds", value = "订单编号列表", required = true, example = "E1024,E2048", dataTypeClass = String.class)
    public CommonResult<List<OrderPickupBackVO>> getPickUpListByOrderNo(@RequestParam("orderIds") String orderIds) {
        try {
            List<OrderPickupBackVO> list = orderPickupService.getPickUpListByOrderNo(orderIds);
            return success(list);
        } catch (ServiceException e){
            return error(e.getCode(),e.getMessage());
        }
    }

    @GetMapping("/getAllPickUpListByOrderNo")
    @ApiOperation("获取提货日志")
    public CommonResult<List<OrderPickupDO>> getAllPickUpListByOrderNo(@RequestParam("orderNo") String orderNo) {
        List<OrderPickupDO> list = orderPickupService.getAllPickUpListByOrderNo(orderNo);
        return success(list);
    }

    @GetMapping("/page")
    @ApiOperation("获得提货分页")
    public CommonResult<PageResult<OrderPickupBackVO>> getOrderPickupPage(@Valid OrderPickupQueryVO query, PageVO page) {
        PageResult<OrderPickupDO> pageResult = orderPickupService.getOrderPickupPage(query, page);
        return success(OrderPickupConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出提货 Excel")
    @OperateLog(type = EXPORT)
    public void exportOrderPickupExcel(@Valid OrderPickupQueryVO query,
              HttpServletResponse response) throws IOException {
        List<OrderPickupDO> list = orderPickupService.getOrderPickupList(query);
        // 导出 Excel
        List<OrderPickupBackVO> datas = OrderPickupConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "提货.xls", "数据", OrderPickupBackVO.class, datas);
    }
}
