package cn.iocoder.yudao.module.order.controller.app.orderCargoControl;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.limiter.dynamic.DynamicRateLimiter;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.order.dal.dataobject.orderCargoControl.OrderCargoControlDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderCargoControlPick.OrderCargoControlPickDO;
import cn.iocoder.yudao.module.order.dto.OrderCargoControlReleaseInfoDto;
import cn.iocoder.yudao.module.order.service.orderCargoControl.OrderCargoControlService;
import cn.iocoder.yudao.module.order.service.orderCargoControlPick.OrderCargoControlPickService;
import cn.iocoder.yudao.module.order.vo.order.OrderSendSmsReqVO;
import cn.iocoder.yudao.module.order.vo.orderCargoControl.OrderCargoControlBackVO;
import cn.iocoder.yudao.module.order.vo.orderCargoControl.OrderCargoControlCreateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.*;

@Validated
@RestController
@Api(tags = "app-web - 订单控货人信息")
@RequestMapping("/order/order-cargo-control")
public class AppOrderCargoControlController {

    @Autowired
    private OrderCargoControlService orderCargoControlService;
    @Autowired
    private MemberUserService memberUserService;
    @Resource
    private OrderCargoControlPickService orderCargoControlPickService;

    @PostMapping("/create")
    @ApiOperation("创建订单控货人信息")
    @DynamicRateLimiter(base = "#Headers['Authorization']", permits = 1)
    @Idempotent(timeout = 5)
    @PreAuthenticated
    public CommonResult<Long> createOrderCargoControl(@Valid @RequestBody OrderCargoControlCreateReqVO createReqVO) {
        MemberUserDO memberUserDO = memberUserService.getUser(getLoginUserId());
        return success(orderCargoControlService.createOrderCargoControl(createReqVO, memberUserDO));
    }





    @GetMapping("/info/{orderId}")
    @ApiOperation("获得订单控货权转移编辑详情")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<OrderCargoControlBackVO> getOrderCargoControlInfo(@PathVariable("orderId") Long orderId) {
        OrderCargoControlBackVO backVO = orderCargoControlService.getOrderCargoControlInfo(orderId);
        return success(backVO);
    }

    @GetMapping("/controlPassword/{orderId}")
    @ApiOperation(value = "检查是否有设置控货密码", notes = "根据订单id，检查此控货订单的控货人是否有设置控货密码")
    @PreAuthenticated
    public CommonResult<Boolean> checkControlPassword(@PathVariable("orderId") Long orderId) {
        return success( orderCargoControlService.checkControlPassword(orderId));
    }

    @GetMapping("/release/info/{orderId}")
    @ApiOperation("获得控货订单放货编辑详情")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<OrderCargoControlReleaseInfoDto> getOrderCargoControlReleaseInfo(@PathVariable("orderId") Long orderId) {
        OrderCargoControlReleaseInfoDto releaseInfoDto = orderCargoControlService.getOrderCargoControlReleaseInfo(orderId);
        return success(releaseInfoDto);
    }


    @PutMapping("/review/{orderId}")
    @ApiOperation("批量复合订单控货人放货")
    @PreAuthenticated
    public CommonResult<Boolean> reviewOrderCargoControlPick(@PathVariable("orderId") Long orderId) {
        OrderCargoControlDO orderCargoControlDO = orderCargoControlService.getOne(new LambdaQueryWrapper<OrderCargoControlDO>()
                .eq(OrderCargoControlDO::getOrderId, orderId)
                .eq(OrderCargoControlDO::getIsActual, Boolean.TRUE)
                .orderByDesc(OrderCargoControlDO::getId).last("limit 1"));
        if (Objects.isNull(orderCargoControlDO)) {
            throw exception(ORDER_CARGO_CONTROL_NOT_EXISTS);
        }
        MemberUserDO memberUserDO = memberUserService.getUser(SecurityFrameworkUtils.getLoginUserId());
        if (!Objects.equals(orderCargoControlDO.getPhone(), memberUserDO.getMobile())) {
            throw exception(YOU_NO_IS_CARGO_CONTROLLER);
        }
        List<OrderCargoControlPickDO> orderCargoControlPickDOList = orderCargoControlPickService.list(new LambdaQueryWrapper<OrderCargoControlPickDO>()
                .eq(OrderCargoControlPickDO::getOrderId, orderId)
                .eq(OrderCargoControlPickDO::getStatus, 1));
        if (CollectionUtil.isEmpty(orderCargoControlPickDOList)) {
            throw exception(ORDER_CARGO_CONTROL_PICK_NOT_EXISTS);
        }
        for (OrderCargoControlPickDO orderCargoControlPickDO : orderCargoControlPickDOList) {
            orderCargoControlPickService.reviewOrderCargoControlPick(orderCargoControlPickDO);
        }
        return success(Boolean.TRUE);
    }


    @PostMapping("/send-sms-code")
    @ApiOperation(value = "发送手机验证码", notes = "此接口只适用发送场景(4 控货全转移 5 放货验证短信 6 放货成功通知)")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    @PreAuthenticated
    public CommonResult<Boolean> sendSmsCode(@RequestBody @Valid OrderSendSmsReqVO reqVO) {
        MemberUserDO memberUserDO = memberUserService.getUser(getLoginUserId());
        orderCargoControlService.sendSmsCode(getLoginUserId(), reqVO, memberUserDO);
        return success(true);
    }
}
